// singinst.cpp: implementation of the CSingleInstance class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SingleInstance.h"
#include "singinst.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

bool CSingleInstance::s_bClassRegistered = false;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSingleInstance::CSingleInstance(bool bActivatePrevious,
								 LPCTSTR pszName)
	: m_bActivatePrev( bActivatePrevious ),
	  m_hMutex( NULL )
{
	if (pszName)
	{
		::lstrcpyn( m_szName, pszName, MAX_PATH );
	}
	else
	{
		TCHAR szModName[ MAX_PATH ];
		::GetModuleFileName( NULL, szModName, MAX_PATH );
		TCHAR * pszExeName = szModName + ::lstrlen( szModName );
		while (*pszExeName -- != _T('\\'));
		pszExeName += 2;
		::lstrcpy( m_szName, _T("Single Instance Application -- ") );
		::lstrcat( m_szName, pszExeName );
	}
}

CSingleInstance::~CSingleInstance()
{
	if (m_hMutex)
		::CloseHandle( m_hMutex );
	if (s_bClassRegistered)
		::UnregisterClass( m_szName, ::AfxGetInstanceHandle() );
}

bool CSingleInstance::IsFirstInstance()
{
	HANDLE hMutex = ::CreateMutex( NULL, TRUE, m_szName );
	if (!hMutex)
		return true;

	m_hMutex = hMutex;

	if (ERROR_ALREADY_EXISTS == ::GetLastError())
	// another instance of the app exists
	{
		// do we need to activate the previous instance?
		if (m_bActivatePrev)
		{
			CWnd * pPrevWnd = CWnd::FindWindow( m_szName, NULL );
			if (pPrevWnd)
			{
				if (pPrevWnd->IsIconic())
					pPrevWnd->ShowWindow( SW_RESTORE );

				CWnd * pActivePopup = pPrevWnd->GetLastActivePopup();

				pActivePopup->SetForegroundWindow();
			}
			else
				ASSERT(FALSE);   // is we are the second instance, a previous instance should exist!
			                     // this assert is usually caused when you forget to change the CMainFrame
			                     //    class name (cs.lpszClass) in the CMainFrame::PreCreateWindow( ).
		}

		return false;
	}
	else
	// we are the first instance
	{
		// do we want to activate this instance instead of running new ones later?
		if (m_bActivatePrev)
		{
			WNDCLASS wc;

			::ZeroMemory( &wc, sizeof(WNDCLASS) );

			wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
			wc.hInstance = ::AfxGetInstanceHandle( );
			wc.hIcon = ::LoadIcon( wc.hInstance, MAKEINTRESOURCE(IDR_MAINFRAME) );
			wc.hCursor = ::LoadCursor( NULL, IDC_ARROW );
			wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
			wc.lpfnWndProc = ::DefWindowProc;
			wc.lpszClassName = m_szName;

			if (!::AfxRegisterClass( &wc ))
				return false;

			s_bClassRegistered = true;
		}
	}

	return true;
}

LPCTSTR CSingleInstance::GetWndClassName() const
{
	if (!m_bActivatePrev)
		ASSERT(FALSE);   // if you don't want to activate the previous instance, you don't
	                     //   need this function
	return (LPCTSTR) m_szName;
}